@@ -31,6 +31,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include <the_Foundation/thread.h>
 #include <SDL_audio.h>
 #include <SDL_timer.h>
+#include <SDL.h>
 
 #if defined (LAGRANGE_ENABLE_MPG123)
 #   include <mpg123.h>
@@ -191,7 +192,7 @@ static enum iDecoderStatus decodeVorbis_Decoder_(iDecoder *d) {
         int error;
         int consumed;
         d->vorbis = stb_vorbis_open_pushdata(
-            constData_Block(input), size_Block(input), &consumed, &error, NULL);
+            constData_Block(input), (int) size_Block(input), &consumed, &error, NULL);
         if (!d->vorbis) {
             return needMoreInput_DecoderStatus;
         }
@@ -224,7 +225,7 @@ static enum iDecoderStatus decodeVorbis_Decoder_(iDecoder *d) {
         lock_Mutex(&d->input->mtx);
         d->totalInputSize = size_Block(input);
         int error = 0;
-        stb_vorbis *vrb = stb_vorbis_open_memory(constData_Block(input), size_Block(input),
+        stb_vorbis *vrb = stb_vorbis_open_memory(constData_Block(input), (int) size_Block(input),
                                                  &error, NULL);
         if (vrb) {
             d->totalSamples = stb_vorbis_stream_length_in_samples(vrb);
@@ -455,6 +456,8 @@ struct Impl_Player {
     iAVFAudioPlayer * avfPlayer; /* iOS */
 };
 
+static iPlayer *activePlayer_;
+
 iDefineTypeConstruction(Player)
 
 static size_t sampleSize_Player_(const iPlayer *d) {
@@ -655,8 +658,14 @@ void deinit_Player(iPlayer *d) {
 #if defined (iPlatformAppleMobile)
     if (d->avfPlayer) {
         delete_AVFAudioPlayer(d->avfPlayer);
+        if (activePlayer_ == d) {
+            clearNowPlayingInfo_iOS();
+        }
     }
 #endif
+    if (activePlayer_ == d) {
+        activePlayer_ = NULL;
+    }
 }
 
 iBool isStarted_Player(const iPlayer *d) {
@@ -724,6 +733,29 @@ void updateSourceData_Player(iPlayer *d, const iString *mimeType, const iBlock *
     unlock_Mutex(&input->mtx);
 }
 
+size_t sourceDataSize_Player(const iPlayer *d) {
+    lock_Mutex(&d->data->mtx);
+    const size_t size = size_Block(&d->data->data);
+    unlock_Mutex(&d->data->mtx);
+    return size;
+}
+
+static iBool setupSDLAudio_(iBool init) {
+    static iBool isAudioInited_ = iFalse;
+    if (init) {
+        if (SDL_InitSubSystem(SDL_INIT_AUDIO)) {
+            fprintf(stderr, "[SDL] audio init failed: %s\n", SDL_GetError());
+            return iFalse;
+        }
+        isAudioInited_ = iTrue;
+    }
+    else if (isAudioInited_) {
+        SDL_QuitSubSystem(SDL_INIT_AUDIO);
+        isAudioInited_ = iFalse;
+    }
+    return isAudioInited_;
+}
+
 iBool start_Player(iPlayer *d) {
     if (isStarted_Player(d)) {
         return iFalse;
@@ -732,6 +764,7 @@ iBool start_Player(iPlayer *d) {
     if (d->avfPlayer) {
         play_AVFAudioPlayer(d->avfPlayer);
         setNotIdle_Player(d);
+        activePlayer_ = d;
         return iTrue;
     }
 #endif
@@ -741,6 +774,9 @@ iBool start_Player(iPlayer *d) {
     }
     content.output.callback = writeOutputSamples_Player_;
     content.output.userdata = d;
+    if (!setupSDLAudio_(iTrue)) {
+        return iFalse;
+    }
     d->device = SDL_OpenAudioDevice(NULL, SDL_FALSE /* playback */, &content.output, &d->spec, 0);
     if (!d->device) {
         return iFalse;
@@ -749,6 +785,7 @@ iBool start_Player(iPlayer *d) {
     d->decoder->gain = d->volume;
     SDL_PauseAudioDevice(d->device, SDL_FALSE);
     setNotIdle_Player(d);
+    activePlayer_ = d;
     return iTrue;
 }
 
@@ -779,6 +816,7 @@ void stop_Player(iPlayer *d) {
         d->device = 0;
         delete_Decoder(d->decoder);
         d->decoder = NULL;
+        setupSDLAudio_(iFalse);
     }
 }
 
@@ -882,3 +920,7 @@ iString *metadataLabel_Player(const iPlayer *d) {
     }
     return meta;
 }
+
+iPlayer *active_Player(void) {
+    return activePlayer_;
+}
