@@ -38,9 +38,10 @@ enum iFileVersion {
     serializedSidebarState_FileVersion  = 3,
     addedRecentUrlFlags_FileVersion     = 4,
     bookmarkFolderState_FileVersion     = 5,
+    multipleWindows_FileVersion         = 6,
     /* meta */
-    idents_FileVersion = 1, /* version used by GmCerts/idents.lgr */
-    latest_FileVersion = 5,
+    latest_FileVersion = 6, /* used by state.lgr */
+    idents_FileVersion = 1, /* used by GmCerts/idents.lgr */
 };
 
 enum iImageStyle {
@@ -156,8 +157,9 @@ iLocalDef int acceptKeyMod_ReturnKeyBehavior(int behavior) {
 #define bookmark_Icon       "\U0001f516"
 #define folder_Icon         "\U0001f4c1"
 #define file_Icon           "\U0001f5ce"
-#define openTab_Icon        "\u2750"
-#define openTabBg_Icon      "\u2b1a"
+#define openWindow_Icon     "\u2b1a" //"\U0001F5d4"
+#define openTab_Icon        add_Icon
+#define openTabBg_Icon      "\u2750" //"\u2b1a"
 #define openExt_Icon        "\u27a0"
 #define add_Icon            "\u2795"
 #define page_Icon           "\U00010117"
@@ -186,6 +188,8 @@ iLocalDef int acceptKeyMod_ReturnKeyBehavior(int behavior) {
 #define downAngle_Icon      "\ufe40"
 #define photo_Icon          "\U0001f5bc"
 #define fontpack_Icon       "\U0001f520"
+#define package_Icon        "\U0001f4e6"
+#define paperclip_Icon      "\U0001f4ce"
 
 #if defined (iPlatformApple)
 #   define shift_Icon       "\u21e7"
