@@ -61,18 +61,21 @@ void init_Prefs(iPrefs *d) {
     d->pinSplit          = 1;
     d->time24h           = iTrue;
     d->returnKey         = default_ReturnKeyBehavior;
-    d->hoverLink         = iFalse;
+    d->retainTabs        = iTrue;
+    d->hoverLink         = iTrue;
     d->smoothScrolling   = iTrue;
     d->smoothScrollSpeed[keyboard_ScrollType] = 13;
     d->smoothScrollSpeed[mouse_ScrollType]    = 13;
     d->loadImageInsteadOfScrolling = iFalse;
-    d->collapsePreOnLoad      = iFalse;
-    d->openArchiveIndexPages  = iTrue;
-    d->addBookmarksToBottom   = iTrue;
-    d->warnAboutMissingGlyphs = iTrue;
-    d->decodeUserVisibleURLs  = iTrue;
+    d->openDataUrlImagesOnLoad = iFalse;
+    d->collapsePreOnLoad       = iFalse;
+    d->openArchiveIndexPages   = iTrue;
+    d->addBookmarksToBottom    = iTrue;
+    d->warnAboutMissingGlyphs  = iTrue;
+    d->decodeUserVisibleURLs   = iTrue;
     d->maxCacheSize      = 10;
     d->maxMemorySize     = 200;
+    d->maxUrlSize        = 8192;
     setCStr_String(&d->strings[uiFont_PrefsString], "default");
     setCStr_String(&d->strings[headingFont_PrefsString], "default");
     setCStr_String(&d->strings[bodyFont_PrefsString], "default");
@@ -88,6 +91,7 @@ void init_Prefs(iPrefs *d) {
     d->boldLinkLight     = iTrue;
     d->lineWidth         = 38;
     d->lineSpacing       = 1.0f;
+    d->tabWidth          = 8;
     d->bigFirstParagraph = iTrue;
     d->quoteIcon         = iTrue;
     d->centerShortDocs   = iTrue;
