@@ -115,6 +115,7 @@ void    setCurrent_Text         (iText *);
 
 void    setDocumentFontSize_Text(iText *, float fontSizeFactor); /* affects all except `default*` fonts */
 void    resetFonts_Text         (iText *);
+void    resetFontCache_Text     (iText *);
 
 int     lineHeight_Text         (int fontId);
 float   emRatio_Text            (int fontId); /* em advance to line height ratio */
@@ -227,6 +228,8 @@ struct Impl_WrapText {
 iTextMetrics    measure_WrapText    (iWrapText *, int fontId);
 iTextMetrics    draw_WrapText       (iWrapText *, int fontId, iInt2 pos, int color);
 
+iChar           missing_Text        (size_t index);
+void            resetMissing_Text   (iText *);
 iBool           checkMissing_Text   (void); /* returns the flag, and clears it */
 SDL_Texture *   glyphCache_Text     (void);
 
